# --- CMake utilities, to deal with f2py interface ---


# Create a pyf file that include
# all 'fortran' signature files (sub-pyf files)
# required by f2py.
#
# Usage:
# write_main_pyf_file(fname)
# where fname must be the name of the generated module
#
# For example to create hysop.f2hysop python module,
# call (in CMakeLists.txt)
# write_main_pyf_file(f2hysop)
# --> create f2hysop.pyf that will be used to generate hysop.f2hysop module.
#
function(write_main_pyf_file filename)

  # First writes filename.pyf.in, according to cmake current config
  # and then generate filename.pyf.
  set(_file ${GENERATED_FORTRAN_FILES_DIR}/${filename}.pyf.in)
  file(WRITE ${_file}
"!    -*- f90 -*-\n
! Generated file - Do not edit.\n
! Note: the context of this file is case sensitive.\n
python module ${F_2_HYSOP} ! in\n
  interface\n")
 file(APPEND ${_file} 
      "      ! Example
      include '@GENERATED_FORTRAN_FILES_DIR@/precision.pyf'\n")
 file(APPEND ${_file} 
      "      ! Example
      include '@CMAKE_SOURCE_DIR@/hysop/common_f/template.pyf'\n")
    if(WITH_FFTW)
      file(APPEND ${_file}
      "      ! fftw
      include '@CMAKE_SOURCE_DIR@/hysop/numerics/fftw_f/fftw2py.pyf'\n")
    endif()
    if(WITH_SCALES)
      file(APPEND ${_file}
      "      ! scales
      include '@CMAKE_SOURCE_DIR@/hysop/scales_f/scales2py.pyf'\n")
    endif()
    if(WITH_EXTRAS)
      file(APPEND ${_file}
      "      ! arnoldi
      include '@CMAKE_SOURCE_DIR@/hysop/numerics/extras_f/arnoldi.pyf'\n")
	endif()
 file(APPEND ${_file} "  end interface\n
end python module ${F_2_HYSOP}")

message(STATUS "Generate ${F_2_HYSOP}.pyf file ...")
configure_file(${_file} ${GENERATED_FORTRAN_FILES_DIR}/${filename}.pyf @ONLY)

endfunction()

