

from hysop.backend.device.codegen.base.opencl_codegen   import OpenClCodeGenerator
from hysop.backend.device.codegen.base.function_codegen import OpenClFunctionCodeGenerator
from hysop.backend.device.codegen.base.variables  import CodegenVariable, CodegenVectorClBuiltin
from hysop.backend.device.opencl.opencl_types      import OpenClTypeGen
from hysop.backend.device.codegen.base.utils      import ArgDict
from hysop.backend.device.codegen.base.statistics import WorkStatistics

class ComputeIndexFunction(OpenClFunctionCodeGenerator):

    def __init__(self,typegen,dim,wrap=None,itype='int'):
        assert dim>0
        args = ArgDict()
        args['idx']  = CodegenVectorClBuiltin('idx', 'int', dim, typegen, add_impl_const=bool(wrap==False))
        args['size'] = CodegenVectorClBuiltin('size', itype, dim, typegen, add_impl_const=True)
        args['wrap'] = CodegenVariable('wrap', 'bool',typegen, add_impl_const=True, value=wrap)

        basename = 'compute_index_{}d'.format(dim)
        if wrap is not None:
            basename += '_' + ('' if wrap else 'no') + 'wrap'
        super(ComputeIndexFunction,self).__init__(basename=basename,ext='.c', output=itype,args=args,typegen=typegen,inline=True)
        self.dim = dim
        self.wrap = wrap
        self.itype = itype

        self.gencode()

    def gencode(self):
        s = self
        dim = s.dim
        idx   = s.args['idx']
        size  = s.args['size']
        wrap  = s.args['wrap']
        with s._function_():
            with s._if_(wrap()):
                self.append('{idx} = ({idx}+{size}) % {size};'.format(idx=idx(),size=size))
            ss = '{}'.format(idx[dim-1])
            for i in range(dim-2, -1, -1):
                ss = '({}*{}+{})'.format(ss,size[i],idx[i])
            ret = 'return {};'.format(ss)
            self.append(ret)

    def per_work_statistics(self):
        dim   = self.dim
        wrap  = self.wrap
        itype = self.itype

        ops = {}
        ops[itype] = int(wrap)*2*dim + 2*(dim-1)*dim

        stats = WorkStatistics()
        stats.ops_per_type = ops

        return stats


if __name__ == '__main__':
    from hysop.backend.device.codegen.base.test import test_typegen
    tg = test_typegen('float')
    cg = OpenClCodeGenerator('main',tg)

    ci0 = ComputeIndexFunction(tg,3,wrap=None)
    ci1 = ComputeIndexFunction(tg,3,wrap=True)
    ci2 = ComputeIndexFunction(tg,3,wrap=False)

    cg.require(ci0.name, ci0)
    cg.require(ci1.name, ci1)
    cg.require(ci2.name, ci2)

    cg.edit()

