from hysop.backend.device.codegen.symbolic.expr import TypedI
from hysop.symbolic.misc import BroadCast, Expand, Select

class OpenClBroadCast(TypedI, BroadCast):
    """ v=v.xy => v.xyxy"""
    def __new__(cls, ctype, expr, factor):
        obj = super(OpenClBroadCast, cls).__new__(cls, expr, factor)
        obj.ctype = cls.vtype(expr.basetype, factor*expr.components)
        return obj

    def _ccode(self, printer):
        expr = self.expr
        val = printer._print(expr)
        if (self.factor>1):
            if (self.expr.components>1):
                indices = tuple(i for j in range(self.factor) for i in range(expr.components))
                mode=('hex' if (expr.components>4) else 'pos')
                bc = '({}).{}{}'.format(val,
                        printer.typegen.vtype_access(indices[0], expr.components, mode),
                        ''.join(printer.typegen.vtype_component_adressing(i, mode) for i in indices[1:]))
            else:
                bc = '(({})({}))'.format(self.ctype, val)
        else:
            bc = val
        return bc

class OpenClExpand(TypedI, Expand):
    """ v=v.xy => v.xxyy"""
    def __new__(cls, ctype, expr, factor):
        obj = super(OpenClExpand, cls).__new__(cls, expr, factor)
        obj.ctype = cls.vtype(expr.basetype, factor*expr.components)
        return obj

    def _ccode(self, printer):
        expr = self.expr
        val = printer._print(expr)
        if (self.factor>1):
            if (self.expr.components>1):
                indices = tuple(i for i in range(expr.components) for j in range(self.factor))
                mode=('hex' if (expr.components>4) else 'pos')
                bc = '({}).{}{}'.format(val,
                        printer.typegen.vtype_access(indices[0], expr.components, mode),
                        ''.join(printer.typegen.vtype_component_adressing(i, mode) for i in indices[1:]))
            else:
                bc = '(({})({}))'.format(self.ctype, val)
        else:
            bc = val
        return bc

class OpenClSelect(TypedI, Select):
    def __new__(cls, ctype, a, b, c):
        obj = super(OpenClSelect, cls).__new__(cls, a, b, c)
        assert a.ctype == b.ctype, '{} != {}'.format(a.ctype, b.ctype)
        obj.ctype = a.ctype
        return obj

    def _ccode(self, printer):
        return 'select({}, {}, {})'.format(printer._print(self.a),
                                           printer._print(self.b),
                                           printer._print(self.c))
