import numpy as np

from hysop.tools.types import check_instance
from hysop.backend.device.codegen.base.union_codegen import UnionCodeGenerator
from hysop.backend.device.opencl.opencl_types  import OpenClTypeGen

class FloatIntegerUnion(UnionCodeGenerator):
    def __init__(self, ftype, typegen, typedef=None):

        name,dtype,comments = self.build_dtype(typegen, ftype)

        super(FloatIntegerUnion,self).__init__(name=name,
                dtype=dtype,
                typegen=typegen,
                typedef=typedef,
                comments=comments)

    @staticmethod
    def build_dtype(typegen, ftype):
        tg = typegen

        name = 'float_int'

        dtype = []
        if ftype == 'half':
            name+='16'
            dtype.append(('intval', np.int16))
            dtype.append(('uintval', np.uint16))
            dtype.append(('floatval', np.float16))
        elif ftype == 'float':
            name+='32'
            dtype.append(('intval', np.int32))
            dtype.append(('uintval', np.uint32))
            dtype.append(('floatval', np.float32))
        elif ftype == 'double':
            name+='64'
            dtype.append(('intval', np.int64))
            dtype.append(('uintval', np.uint64))
            dtype.append(('floatval', np.float64))
        else:
            msg='Unknown ftype \'{}\', only half, float and double are supported.'
            msg=msg.format(ftype)
            raise ValueError(msg)

        comments = [
                'Access value as a signed integer',
                'Access value as a unsigned integer',
                'Access value as a floating point',
        ]

        return name, dtype, comments


if __name__ == '__main__':
    from hysop.backend.device.codegen.base.opencl_codegen import OpenClCodeGenerator
    from hysop.backend.device.codegen.base.test import _test_typegen

    tg = _test_typegen()

    u1 = FloatIntegerUnion('double', tg)
    u2 = FloatIntegerUnion('float', tg, 'custom32')

    cg = OpenClCodeGenerator('test_generator',tg)
    cg.declare_cl_extension('cl_khr_fp64')
    cg.require('u1',u1)
    cg.require('u2',u2)

    cg.edit()

    cg.test_compile()
