
from abc import ABCMeta, abstractmethod

class Platform(object, metaclass=ABCMeta):

    def __init__(self, hardware_topo, platform_handle, platform_id, **kwds):
        super(Platform, self).__init__(**kwds)
        self._platform_id = platform_id
        self._logical_devices = {}
        self._discover_devices(hardware_topo, platform_handle)
        # we do not keep a reference to platform_handle as we
        # need to pickle this object

    @property
    def platform_id(self):
        return self._platform_id

    @property
    def logical_devices(self):
        return self._logical_devices

    @property
    def physical_devices(self):
        return [dev.physical_device() for dev in self.logical_devices()]

    @abstractmethod
    def _discover_devices(self, hardware_topo, platform_handle):
        pass
