from hysop.tools.decorators import debug
from hysop.core.graph.graph import op_apply
from hysop.operator.base.min_max import MinMaxFieldStatisticsBase, \
                                        MinMaxDerivativeStatisticsBase
from hysop.backend.host.host_operator import HostOperator
from hysop.backend.host.python.operator.derivative import PythonSpectralSpaceDerivative, \
        PythonFiniteDifferencesSpaceDerivative


class PythonMinMaxFieldStatistics(MinMaxFieldStatisticsBase, HostOperator):
    """Python implementation backend of operator MinMaxFieldStatistics."""

    @debug
    def __new__(cls, **kwds):
        return super(PythonMinMaxFieldStatistics, cls).__new__(cls, **kwds)

    @debug
    def __init__(self, **kwds):
        """See MinMaxFieldStatisticsBase.__init__()."""
        super(PythonMinMaxFieldStatistics, self).__init__(**kwds)

    @op_apply
    def apply(self, **kwds):
        """See MinMaxFieldStatisticsBase.apply()."""
        super(PythonMinMaxFieldStatistics, self).apply(**kwds)
        self.compute_statistics(**kwds)

    @classmethod
    def supports_mpi(cls):
        return True


class PythonMinMaxSpectralDerivativeStatistics(MinMaxDerivativeStatisticsBase,
                                               PythonSpectralSpaceDerivative):
    """Python implementation backend of operator MinMaxSpectralDerivativeStatistics."""
    @op_apply
    def apply(self, **kwds):
        """Compute derivative and then statistics."""
        super(PythonMinMaxSpectralDerivativeStatistics, self).apply(**kwds)
        self.compute_statistics(**kwds)


class PythonMinMaxFiniteDifferencesDerivativeStatistics(MinMaxDerivativeStatisticsBase,
                                                        PythonFiniteDifferencesSpaceDerivative):
    """Python implementation backend of operator MinMaxFiniteDifferencesDerivativeStatistics."""
    @op_apply
    def apply(self, **kwds):
        """Compute derivative and then statistics."""
        super(PythonMinMaxFiniteDifferencesDerivativeStatistics, self).apply(**kwds)
        self.compute_statistics(**kwds)

    @classmethod
    def supports_mpi(cls):
        return True
