"""A list of authorized keys that may be used to set methods
in operators.

Usage
------

method = {key: value, ...}

Key must be one of the constants given below. Value is usually a class name. 
See details in each operator.

You can fetch available operator method keys by using operator.available_methods().
This will return a distionnary with all the possible configurations.

Default methods can be retrived with operator.default_methods()

Example
--------
For the stretching case, you may use:

.. code::

    method = {TimeIntegrator: RK3, Formulation: Conservative,
              SpaceDiscretization: FDC4}

"""
from hysop.backend import __HAS_OPENCL_BACKEND__

from hysop.constants import Backend, Precision, BoundaryCondition, \
                            TimeIntegratorsOptimisationLevel, FieldProjection, \
                            StretchingFormulation, SpaceDiscretization, \
                            ComputeGranularity

from hysop.operator.spatial_filtering import FilteringMethod
from hysop.numerics.interpolation.interpolation import Interpolation, MultiScaleInterpolation
from hysop.numerics.interpolation.polynomial import PolynomialInterpolator, PolynomialInterpolation
from hysop.numerics.odesolvers.runge_kutta import TimeIntegrator
from hysop.numerics.remesh.remesh import Remesh
from hysop.numerics.splitting.strang import StrangOrder

if __HAS_OPENCL_BACKEND__:
    from hysop.backend.device.opencl.opencl_kernel_config import OpenClKernelConfig
    from hysop.backend.device.opencl.opencl_kernel_autotuner_config import \
            OpenClKernelAutotunerConfig
