"""
Dummy operator.
To be used for topology fixing.
"""
from hysop.core.graph.computational_node_frontend import ComputationalGraphNodeFrontend
from hysop.constants import Implementation
from hysop.backend.host.host_operator import HostOperator
from hysop.tools.types import check_instance
from hysop.topology.cartesian_descriptor import CartesianTopologyDescriptors
from hysop.fields.continuous_field import Field
from hysop.tools.decorators import debug
from hysop.core.graph.graph import op_apply


class PythonDummy(HostOperator):

    @debug
    def __new__(cls, variables, **kwds):
        return super(PythonDummy, cls).__new__(cls,
            input_fields=None, output_fields=None, **kwds)

    @debug
    def __init__(self, variables, **kwds):
        check_instance(variables, dict, keys=Field,
                       values=CartesianTopologyDescriptors)
        input_fields = variables
        output_fields = variables
        super(PythonDummy, self).__init__(
            input_fields=input_fields, output_fields=output_fields,
            **kwds)

    @debug
    def get_field_requirements(self):
        requirements = super(PythonDummy, self).get_field_requirements()
        for (is_input, reqs) in requirements.iter_requirements():
            if (reqs is None):
                continue
            (field, td, req) = reqs
            req.axes = (tuple(range(field.dim)), )
        return requirements

    @op_apply
    def apply(self, **kwds):
        super(PythonDummy, self).apply(**kwds)
        # Here doing nothing

    @classmethod
    def supports_mpi(cls):
        return True


class Dummy(ComputationalGraphNodeFrontend):

    __implementations = {
        Implementation.PYTHON: PythonDummy
    }

    @classmethod
    def implementations(cls):
        return cls.__implementations

    @classmethod
    def default_implementation(cls):
        return Implementation.PYTHON

