
import sympy as sm
from hysop.tools.numerics import is_complex

class ComplexExpr(sm.Expr):
    pass

class ComplexUnaryExpr(ComplexExpr):
    def __new__(cls, a, **kwds):
        obj = super(ComplexUnaryExpr, cls).__new__(cls, a, **kwds)
        obj.a = a
        return obj
    def __repr__(self):
        return '{}({})'.format(self.__class__.__name__, repr(self.a))
    def __str__(self):
        return '{}({})'.format(self.fname, self.a)
    def _sympystr(self, printer):
        return '{}({})'.format(self.fname, printer._print(self.a))

class ComplexBinaryExpr(ComplexExpr):
    def __new__(cls, lhs, rhs, **kwds):
        obj = super(ComplexBinaryExpr, cls).__new__(cls, lhs, rhs, **kwds)
        obj.lhs, obj.rhs = lhs, rhs
        return obj
    def __repr__(self):
        return '{}({},{})'.format(self.__class__.__name__, repr(self.lhs), repr(self.rhs))
    def __str__(self):
        return '{}({},{})'.format(self.fname, self.rhs, self.rhs)
    def _sympystr(self, printer):
        return '{}({},{})'.format(self.fname, printer._print(self.lhs), printer._print(self.rhs))

class ComplexMul(ComplexBinaryExpr):
    """Tag for complex multiplication."""
    fname='cmul'

class ComplexModulus(ComplexUnaryExpr):
    """Tag for complex modulus."""
    fname='cabs'

class ComplexModulus2(ComplexUnaryExpr):
    """Tag for squared complex modulus."""
    fname='cabs2'

def cabs(a):
    return ComplexModulus(a) if is_complex(a) else sm.Abs(a)

def cabs2(a):
    return ComplexModulus2(a) if is_complex(a) else a*a
