
from hysop.tools.numpywrappers import npw
from hysop.symbolic.base import DummySymbolicScalar, sm

class SymbolicConstant(DummySymbolicScalar):
    """Temporary constant variables."""
    def __new__(cls, name, **kwds):
        if ('dtype' not in kwds):
            msg='dtype has not been specified for SymbolicConstant {}.'
            msg=msg.format(name)
            raise RuntimeError(msg)
        dtype = kwds.pop('dtype')
        value = kwds.pop('value', None)
        obj = super(SymbolicConstant, cls).__new__(cls, name=name, **kwds)
        obj._dtype = dtype
        obj._value = None
        if (value is not None):
            obj.bind_value(value)
        return obj

    @property
    def is_bound(self):
        return (self._value is not None)

    def assert_bound(self):
        if not self.is_bound:
            msg='{}::{} value has not been bound yet.'
            msg=msg.format(self.__class__.__name__, self.name)
            raise RuntimeError(msg)

    def bind_value(self, value, force=False):
        if (not force) and (self._value is not None):
            msg='A value has already been bound to SymbolicConstant {}.'.format(self.name)
            raise RuntimeError(msg)
        if isinstance(value, npw.ndarray):
            assert value.size == 1, value.size
            value = value.item()
        value = self._dtype(value)
        self._value = value
        return self

    @property
    def value(self):
        self.assert_bound()
        return self._value
    @property
    def dtype(self):
        self.assert_bound()
        return self._dtype
    @property
    def ctype(self):
        from hysop.backend.device.codegen.base.variables import dtype_to_ctype
        self.assert_bound()
        return dtype_to_ctype(self._dtype)

    def short_description(self):
        self.assert_bound()
        return '{}[dtype={}, ctype={}]'.format(
                self.__class__.__name__, self.dtype, self.ctype)

    def __eq__(self, other):
        return id(self) == id(other)
    def __hash__(self):
        return id(self)
    def _hashable_content(self):
        """See sympy.core.basic.Basic._hashable_content()"""
        hc = super(SymbolicConstant, self)._hashable_content()
        hc += (str(id(self)),)
        return hc
