
from hysop.symbolic.base import SymbolicTensor, SymbolicScalar
from hysop.tools.types import check_instance, to_tuple, first_not_None
from hysop.tools.numpywrappers import npw

class SymbolicTensorParameter(SymbolicTensor):
    def __new__(cls, parameter, name=None, pretty_name=None,
            value=None, shape=None, scalar_cls=None, **kwds):
        from hysop.parameters.tensor_parameter import TensorParameter
        check_instance(parameter, TensorParameter)
        value = first_not_None(value, parameter)
        shape = first_not_None(shape, parameter.shape)
        name  = first_not_None(name,  parameter.name)
        pname = first_not_None(pretty_name, parameter.pretty_name)
        scalar_cls = first_not_None(scalar_cls, SymbolicScalarParameter)
        scalar_kwds = dict(parameter=parameter)
        def make_scalar_kwds(idx):
            return dict(view=lambda parameter: parameter._value[idx])
        return super(SymbolicTensorParameter, cls).__new__(cls,
                name=name, pretty_name=pname,
                shape=shape, value=value, scalar_cls=scalar_cls,
                make_scalar_kwds=make_scalar_kwds,
                scalar_kwds=scalar_kwds, **kwds)

    def __init__(self, parameter, name=None, pretty_name=None,
            value=None, shape=None, scalar_cls=None, **kwds):
        super(SymbolicTensorParameter, self).__init__(
                name=name, pretty_name=pretty_name,
                shape=shape, value=value, scalar_cls=scalar_cls,
                make_scalar_kwds=None,
                scalar_kwds=None, **kwds)

class SymbolicScalarParameter(SymbolicScalar):
    def __new__(cls, parameter, name=None, pretty_name=None, value=None, **kwds):
        from hysop.parameters.tensor_parameter import TensorParameter
        check_instance(parameter, TensorParameter)
        value = first_not_None(value, parameter._value)
        name  = first_not_None(name, parameter.name)
        pname = first_not_None(pretty_name, parameter.pretty_name)
        obj = super(SymbolicScalarParameter, cls).__new__(cls,
                name=name, pretty_name=pname,
                value=value, **kwds)
        obj.parameter = parameter
        return obj

    def __init__(self, parameter, name=None, pretty_name=None, value=None, **kwds):
        super(SymbolicScalarParameter, self).__init__(name=name, pretty_name=pretty_name,
                value=value, **kwds)


if __name__ == '__main__':
    import sympy as sm
    from hysop.parameters.tensor_parameter import TensorParameter
    from hysop.parameters.scalar_parameter import ScalarParameter
    a = ScalarParameter('A', dtype=npw.int32, initial_value=4)
    b = TensorParameter('B', shape=(3,), dtype=npw.int32, initial_value=8)
    c = TensorParameter('C', shape=(3,3), dtype=npw.float32, initial_value=12)
    A = a.s
    B = b.s
    C = c.s
    D = C.copy()
    with D.write_context():
        D[0,0] = B[0]
        D[1,1] = B[1]
        D[2,2] = B[2]

    print(A)
    print(B)
    print(C)
    print(D)
    print()
    print(A.vreplace())
    print(B.vreplace())
    print(C.vreplace())
    print(D.vreplace())
    print()
    a.set_value(1)
    b.set_value(npw.asarray([-1,0,1], dtype=b.dtype))
    c.set_value(npw.full_like(c.value, 3))
    print(A)
    print(B)
    print(C)
    print(D)
    print()
    print(A.vreplace())
    print(B.vreplace())
    print(C.vreplace())
    print(D.vreplace())
    print()
