
from hysop.constants import SpaceDiscretization
from hysop.tools.types import check_instance, to_list, first_not_None, InstanceOf
from hysop.tools.decorators import debug
from hysop.numerics.interpolation.polynomial import PolynomialInterpolator, PolynomialInterpolation

class SpaceDiscretizationMethod(object):
    """
    Operator helper to handle space discretization method.
    """
    __default_method = {
        SpaceDiscretization: 2
    }

    __available_methods = {
        SpaceDiscretization: (InstanceOf(int), InstanceOf(SpaceDiscretization)),
    }

    @classmethod
    def default_method(cls):
        dm = super(SpaceDiscretizationMethod, cls).default_method()
        dm.update(cls.__default_method)
        return dm

    @classmethod
    def available_methods(cls):
        am = super(SpaceDiscretizationMethod, cls).available_methods()
        am.update(cls.__available_methods)
        return am

    @debug
    def handle_method(self,method):
        super(SpaceDiscretizationMethod, self).handle_method(method)
        sdm = method.pop(SpaceDiscretization)
        if isinstance(sdm, int):
            sd = sdm 
        elif isinstance(sdm, SpaceDiscretization):
            assert str(sdm).startswith('FDC')
            sd = int(str(sdm)[3:])
        else:
            raise NotImplementedError(sdm)
        self.space_discretization_method = sdm
        self.space_discretization = sd


class PolynomialInterpolationMethod(SpaceDiscretizationMethod):
    """
    Operator helper to handle polynomial interpolation method.
    """
    __default_method = {
        PolynomialInterpolator: PolynomialInterpolation.LINEAR,
    }

    __available_methods = {
        PolynomialInterpolator: InstanceOf(PolynomialInterpolation),
    }

    @classmethod
    def default_method(cls):
        dm = super(PolynomialInterpolationMethod, cls).default_method()
        dm.update(cls.__default_method)
        return dm

    @classmethod
    def available_methods(cls):
        am = super(PolynomialInterpolationMethod, cls).available_methods()
        am.update(cls.__available_methods)
        return am

    @debug
    def handle_method(self,method):
        super(PolynomialInterpolationMethod, self).handle_method(method)
        fd = self.space_discretization
        pi = method.pop(PolynomialInterpolator)
        self.polynomial_interpolation_method = pi
        self.polynomial_interpolator = PolynomialInterpolator.build_interpolator(pi=pi, fd=fd, dim=self.dim)


