
from abc import ABCMeta
from hysop.tools.enum import EnumFactory
from hysop.tools.types import check_instance, first_not_None
from hysop.tools.decorators import debug

Variable = EnumFactory.create('Variable', ['DISCRETE_FIELD', 'PARAMETER'])

class VariableTag(object, metaclass=ABCMeta):
    """Tag for HySoP variables."""

    @debug
    def __new__(cls, variable_kind=None, **kwds):
        check_instance(variable_kind, Variable, allow_none=True)
        obj = super(VariableTag, cls).__new__(cls, **kwds)
        obj.__variable_kind = variable_kind
        return obj

    @debug
    def __init__(self, variable_kind=None, **kwds):
        check_instance(variable_kind, Variable, allow_none=True)
        super(VariableTag, self).__init__(**kwds)
        self.__variable_kind = first_not_None(self.__variable_kind, variable_kind)
        assert (self.__variable_kind is not None), 'variable_kind has not been setup correctly.'

    def _get_variable_kind(self):
        """Get variable kind."""
        return self.__variable_kind

    kind = property(_get_variable_kind)
